<?php

namespace Drupal\entity_reference_patterns\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\Entity\DraggableListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;

/**
 * Entity Reference patterns list.
 */
class EntityReferencePatternListBuilder extends DraggableListBuilder {

  /**
   * {@inheritdoc}
   */
  protected $limit = FALSE;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_reference_pattern_list';
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['label'] = $this->t('Label');
    $header['pattern'] = $this->t('Pattern');
    $header['pattern_type'] = $this->t('Pattern type');
    $header['conditions'] = $this->t('Conditions');
    $header['status'] = $this->t('Status');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['label'] = $entity->label();
    $row['pattern']['#markup'] = $entity->getPattern();
    $row['pattern_type']['#markup'] = $entity->getEntityTypeLabel();
    $row['conditions']['#theme'] = 'item_list';
    foreach ($entity->getSelectionCriteria() as $type => $condition) {
      if ($type == 'bundles') {
        foreach ($condition as $key => $val) {
          $row['conditions']['#items'][] = $this->t('The %type bundle is %bundle', ['%type' => $entity->getType(), '%bundle' => $key]);
        }
      }
    }
    $row['status']['#markup'] = Markup::create('<span class="views-field">' . ($entity->get('status')
      ? '<span class="marker marker--published">' . $this->t('Enabled') . '</span>'
      : '<span class="marker">' . $this->t('Disabled') . '</span>') . '</span>');
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $operations = parent::getDefaultOperations($entity);
    $ajax_attributes = [
      'class' => ['use-ajax'],
      'data-dialog-type' => 'modal',
      'data-dialog-options' => Json::encode(['width' => 530]),
    ];
    if ($entity->access('duplicate') ) {
      $operations['duplicate'] = [
        'title' => $this->t('Duplicate'),
        'weight' => 0,
        'url' => $this->ensureDestination($entity->toUrl('duplicate')),
        'attributes' => $ajax_attributes,
      ];
    }
    if ($entity->access('update') ) {
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'weight' => 0,
        'url' => $this->ensureDestination($entity->toUrl('edit')),
        'attributes' => $ajax_attributes,
      ];
    }
    if ($entity->access('delete') ) {
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'weight' => 0,
        'url' => $this->ensureDestination($entity->toUrl('delete')),
        'attributes' => $ajax_attributes,
      ];
    }
    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->messenger()->addStatus($this->t('The configuration options have been saved.'));
    parent::submitForm($form, $form_state);
  }

  /**
   * Sets the weight of an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity whose weight will be set.
   * @param int|float $weight
   *   The weight to set on the entity.
   *
   * @return $this
   */
  protected function setWeight(EntityInterface $entity, int|float $weight): EntityInterface {
    /** @var \Drupal\Core\Config\Entity\ConfigEntityInterface $entity */
    $entity->set($this->weightKey, $weight);
    return $entity;
  }

}
