<?php

namespace Drupal\entity_reference_patterns;

use Drupal\Core\Entity\EntityAutocompleteMatcher;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Tags;
use Drupal\Core\Entity\EntityReferenceSelection\SelectionPluginManagerInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Utility\Token;
use Drupal\entity_reference_patterns\Entity\EntityReferencePatternEntity;

class EntityReferencePatternMatcher extends EntityAutocompleteMatcher {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * The token manager.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * Constructs an EntityAutocompleteMatcher object.
   *
   * @param \Drupal\Core\Entity\EntityReferenceSelection\SelectionPluginManagerInterface $selection_manager
   *   The entity reference selection handler plugin manager.
   */
  public function __construct(SelectionPluginManagerInterface $selection_manager, EntityTypeManagerInterface $entity_type_manager, EntityRepositoryInterface $entity_repository, Token $token) {
    parent::__construct($selection_manager);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityRepository = $entity_repository;
    $this->token = $token;
  }

  /**
   * Gets matched labels based on a given search string.
   */
  public function getMatches($target_type, $selection_handler, $selection_settings, $string = '') {
    $entity_ids = $this->entityTypeManager->getStorage('entity_reference_pattern')->getQuery()
      ->accessCheck(TRUE)
      ->condition('type', $target_type)
      ->condition('status', 1)
      ->sort('weight', 'DESC')
      ->range(0, 1)
      ->execute();

    if ($labels = EntityReferencePatternEntity::loadMultiple($entity_ids)) {
      $label_config_entity = reset($labels);
      $criteria = $label_config_entity->getSelectionCriteria();
      $matches = [];

      $options = $selection_settings + [
        'target_type' => $target_type,
        'handler' => $selection_handler,
        'handler_settings' => $selection_settings,
      ];

      $handler = $this->selectionManager->getInstance($options);

      if (!empty($string)) {
        // Get an array of matching entities.
        $match_operator = !empty($selection_settings['match_operator']) ? $selection_settings['match_operator'] : 'CONTAINS';
        $entity_labels = $handler->getReferenceableEntities($string, $match_operator, 10);

        // Loop through the entities and convert them into autocomplete output.
        foreach ($entity_labels as $values) {
          foreach ($values as $entity_id => $label) {
            $entity = $this->entityTypeManager->getStorage($target_type)->load($entity_id);
            $entity = $this->entityRepository->getTranslationFromContext($entity);

            if (!empty($criteria['bundles'])) {
              foreach ($criteria as $type => $condition) {
                if ($type == 'bundles') {
                  foreach ($condition as $k => $v) {
                    if ($k == $entity->bundle()) {
                      $label = $this->token->replace(
                        $label_config_entity->getPattern(),
                        [$entity->getEntityType()->id() => $entity],
                        ['clear' => TRUE]
                      );
                    }
                  }
                }
              }
            }
            else {
              $label = $this->token->replace(
                $label_config_entity->getPattern(),
                [$entity->getEntityType()->id() => $entity],
                ['clear' => TRUE]
              );
            }

            // Just to be safe here we replace ([digital]) petterns so referenced ID could be extracted.
            $label = preg_replace("/\(([\d\h,]+)\)/", "[$1]", $label);

            $key = $label . ' (' . $entity_id . ')';
            // Strip things like starting/trailing white spaces, line breaks and tags.
            $key = preg_replace('/\s\s+/', ' ', str_replace("\n", '', trim(Html::decodeEntities(strip_tags($key)))));
            // Names containing commas or quotes must be wrapped in quotes.
            $key = Tags::encode($key);

            $matches[] = ['value' => $key, 'label' => $label];
          }
        }
      }

      return $matches;
    }
    else {
      return parent::getMatches($target_type, $selection_handler, $selection_settings, $string);
    }
  }

}
